//+------------------------------------------------------------------+
//|                                  Bands Envelopes Know Sure Thing |
//|                                      Copyright 2023 © Shumer3000 |
//+------------------------------------------------------------------+
#property copyright "Copyright © 2023"
#property link      "https://www.mql5.com/ru/users/shumer3000"
#property description "BEKST"
#property strict
//-----
#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color1  Blue 
#property indicator_color2  Red  
#property indicator_color3  Blue 
#property indicator_color4  Red   
#property indicator_color5  Blue 
#property indicator_color6  Red  
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 2
#property indicator_width4 2
#property indicator_width5 1
#property indicator_width6 1 
//---- input parameters
extern int f1=9;          //fast period ROC1
extern int f2=12;         //fast period ROC2
extern int f3=18;         //fast period ROC3
extern int f4=24;         //fast period ROC4
extern int s1=26;         //slow period ROC1..3
extern int s2=39;         //slow period ROC4
extern int fp=26;         //fast period filter
extern int sp=5;          //slow period filter
extern int mp=13;         //period MA
extern double dev1=0.05;  //KST deviation
extern double dev2=2.6;   //BAND deviation
extern bool al=false;     //Alert ON/OFF
//---- buffers
double KST[];
double UKST[];
double DKST[];
double HKST[];
double LKST[];
double SHKST[];
double SLKST[];
double MA1[];
double MA2[];
double MA3[];
double MA4[];
double EMA1[];
double EMA2[];
double EMA3[];
double EMA4[];
double SU[];
double SD[];
//----
double MyPoint,hdev,ldev,d10,d11,d12,d20,d21,d30,d31,ma0,ma1;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
   IndicatorShortName("BEKST("+dev1+","+dev2+")");
   IndicatorBuffers(17);
   SetIndexBuffer(0,UKST);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexLabel(0,"UKST");
   SetIndexBuffer(1,DKST);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexLabel(1,"DKST");
   SetIndexBuffer(2,SHKST);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexLabel(2,"SHKST");
   SetIndexBuffer(3,SLKST);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexLabel(3,"SLKST");
   SetIndexBuffer(4,SU);
   SetIndexStyle(4,DRAW_ARROW,0,1);
   SetIndexArrow(4,233);
   SetIndexBuffer(5,SD);
   SetIndexStyle(5,DRAW_ARROW,0,1);
   SetIndexArrow(5,234);
   SetIndexBuffer(6,KST);
   SetIndexBuffer(7,MA1);
   SetIndexBuffer(8,MA2);
   SetIndexBuffer(9,MA3);
   SetIndexBuffer(10,MA4);
   SetIndexBuffer(11,EMA1);
   SetIndexBuffer(12,EMA2);
   SetIndexBuffer(13,EMA3);
   SetIndexBuffer(14,EMA4);
   SetIndexBuffer(15,HKST);
   SetIndexBuffer(16,LKST);
//----    
	if (Digits == 5 || (Digits == 3 && StringFind(Symbol(), "JPY") != -1))
     MyPoint = Point*10;
	else if (Digits == 6 || (Digits == 4 && StringFind(Symbol(), "JPY") != -1))
     MyPoint = Point*100;
	else
	  MyPoint = Point;
//----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()  {
   return(0);
}
//+------------------------------------------------------------------+
//| Ticks Flow Index                                                 |
//+------------------------------------------------------------------+
double TFI(int tftf, int lperiod, int speriod, int shift, double& stf)  
  {
   double tu,td,ltf;
   stf=0;
   for(int j=speriod-1; j>=0; j--) {
     double ltu=0,ltd=0,wsum=0;
     for(int y=lperiod+shift; y>shift; y--) {
        tu=(iVolume(NULL,tftf,y+j)+(iClose(NULL,tftf,y+j)-iOpen(NULL,tftf,y+j))/Point)/2;
        td=iVolume(NULL,tftf,y+j)-tu;
        ltu+=tu*(y-shift); 
        ltd+=td*(y-shift);
        wsum+=y-shift; 
     }
     ltf=(ltu-ltd)/wsum; 
     stf+=ltf;
   }
   stf/=speriod; 
   return(ltf);
  }
//+------------------------------------------------------------------+
//| Bands Envelopes Know Sure Thing                                  |
//+------------------------------------------------------------------+
int start() {
   int i,counted_bars=IndicatorCounted();
   //---- check for possible errors
   if(counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   int limit=Bars-counted_bars-1;
   //----
   for(i=limit; i>=0; i--) {
      MA1[i]=iMA(NULL,0,f1,0,MODE_EMA,PRICE_CLOSE,i);
      MA2[i]=iMA(NULL,0,f2,0,MODE_EMA,PRICE_CLOSE,i);
      MA3[i]=iMA(NULL,0,f3,0,MODE_EMA,PRICE_CLOSE,i);
      MA4[i]=iMA(NULL,0,f4,0,MODE_EMA,PRICE_CLOSE,i);
   }   
   //----
   for(i=limit; i>=0; i--) {
      EMA1[i]=iMAOnArray(MA1,0,s1,0,MODE_EMA,i);
      EMA2[i]=iMAOnArray(MA2,0,s1,0,MODE_EMA,i);
      EMA3[i]=iMAOnArray(MA3,0,s1,0,MODE_EMA,i);
      EMA4[i]=iMAOnArray(MA4,0,s2,0,MODE_EMA,i);
   } 
   //----
   for(i=limit; i>=0; i--) 
      KST[i]=(EMA1[i]+2*EMA2[i]+3*EMA3[i]+4*EMA4[i])/10; 
   //----
   if(limit==1) return(0);
   double eh=0.0, el=0.0;
   for(i=Bars-s2-1; i>=0; i--) {
      int pb=0;
      double rh=0.0, rl=0.0;
      for(int j=s2-1;j>=0;j--) 
         if(Volume[j]<Volume[j+1])  {
            hdev=High[i+j]-KST[i+j];
            ldev=KST[i+j]-Low[i+j];
            rh+=MathPow(hdev,2);
            rl+=MathPow(ldev,2);
            pb++;
         } 
      rh=MathSqrt(rh/pb);
      rl=MathSqrt(rl/pb);
      HKST[i]=KST[i]+rh*dev2; 
      LKST[i]=KST[i]-rl*dev2;
      if(hdev>eh) eh=hdev;
      if(ldev>el) el=ldev;
   } 
   //----
   for(i=Bars-s2-1; i>=0; i--) {
      UKST[i]=KST[i]+eh*dev1; 
      DKST[i]=KST[i]-el*dev1; 
      SHKST[i]=iMAOnArray(HKST,0,s2,0,MODE_SMMA,i);
      SLKST[i]=iMAOnArray(LKST,0,s2,0,MODE_SMMA,i);
   }
   //----
   ObjectDelete("Price_Up2");
   ObjectCreate("Price_Up2",OBJ_ARROW_RIGHT_PRICE,0,TimeCurrent(),SHKST[0]);
   ObjectSet("Price_Up2",OBJPROP_COLOR,clrBlue);
   ObjectDelete("Price_Up1");
   ObjectCreate("Price_Up1",OBJ_ARROW_RIGHT_PRICE,0,TimeCurrent(),UKST[0]);
   ObjectSet("Price_Up1",OBJPROP_COLOR,clrBlue);
   ObjectDelete("Price_Dn1");
   ObjectCreate("Price_Dn1",OBJ_ARROW_RIGHT_PRICE,0,TimeCurrent(),DKST[0]);
   ObjectDelete("Price_Dn2");
   ObjectCreate("Price_Dn2",OBJ_ARROW_RIGHT_PRICE,0,TimeCurrent(),SLKST[0]);
   //----
   if(iBars(NULL,15)<fp+sp+s2) return(0);
   d30=TFI(0,3*fp,2*sp,0,d31);
   d20=TFI(0,fp,sp,0,d21);
   d10=TFI(1,fp,sp,0,d11);
   TFI(1,fp,sp,1,d12); 
   ma0=iMA(NULL,0,mp,0,MODE_LWMA,PRICE_CLOSE,0);
   ma1=iMA(NULL,0,mp,0,MODE_LWMA,PRICE_CLOSE,2);
   //----
   if(d30>d31&&d20>d21&&d10>d11&&d11>d12&&ma1<ma0) 
      if((SLKST[2]<SLKST[0]&&UKST[2]<UKST[0]&&ma1<UKST[2]&&Low[1]<UKST[1]&&Close[0]>UKST[1])||       //Up High line Up Envelopes
         (SHKST[2]>=SHKST[0]&&ma1<SHKST[2]&&Low[1]<SHKST[1]&&Close[0]>SHKST[1])||                    //Up High line Down Bands
         (SLKST[2]>SLKST[0]&&ma1<SLKST[2]&&ma0>SLKST[0])||                                           //Up Low line Down Bands
         (SLKST[2]<=SLKST[0]&&SHKST[2]>SHKST[0]&&DKST[2]>DKST[0]&&ma1<DKST[2]&&                      //Up Low line Down Envelopes
         Low[1]<DKST[1]&&Close[0]>DKST[1])) {
         SU[0]=Low[0]; 
         if(al) Alert("Buy Order or close the sale");
      }
   if(d30<d31&&d20<d21&&d10<d11&&d11<d12)  
      if((SHKST[2]>SHKST[0]&&DKST[2]>DKST[0]&&ma1<DKST[2]&&High[1]>DKST[1]&&Close[0]<DKST[1])||      //Down Low line Down Envelopes
         (SLKST[2]<=SLKST[0]&&ma1>SLKST[2]&&High[1]>SLKST[1]&&Close[0]<SLKST[1])||                   //Down Low line Up Bands
         (SHKST[2]<SHKST[0]&&ma1>SHKST[2]&&ma0<SHKST[0])||                                           //Down High line Up Bands
         (SHKST[2]>=SHKST[0]&&SLKST[2]<SLKST[0]&&UKST[2]<UKST[0]&&ma1>UKST[2]&&                      //Down High line Up Envelopes
         High[1]>UKST[1]&&Close[0]<UKST[1])) {
         SD[0]=High[0]; 
         if(al) Alert("Sell Order or close your purchases");
      }
   //----
   return(0);
}
//+------------------------------------------------------------------+